/*
  htmlencode.c
  
   1  2006-03-18  Matlib 
   2  2009-02-22  Matlib  + link against libarguments
                          + add <br> support
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <arguments.h>

#define BUFFER_SIZE 128

#define CHECK_ERROR(__COND__,__FUNC__) \
  { \
    if (__COND__) \
    { \
      perror (__FUNC__); \
      exit (EXIT_FAILURE); \
    } \
  }
     


struct Argument arguments [] =
{
  { 'b', "br",         argument_bool, ARGUMENT_F_ONCE, NULL },
  { 'B', "xml-br",     argument_bool, ARGUMENT_F_ONCE, NULL },
  { 'c', "leave-cr",   argument_bool, ARGUMENT_F_ONCE, NULL },
  { 'l', "no-lf",      argument_bool, ARGUMENT_F_ONCE, NULL },
  { 's', "spaces",     argument_bool, ARGUMENT_F_ONCE, NULL },
  { 'S', "all-spaces", argument_bool, ARGUMENT_F_ONCE, NULL },
  { }
};
#define ARG_BR  (arguments [0].value != NULL)
#define ARG_BRX (arguments [1].value != NULL)
#define ARG_CR  (arguments [2].value != NULL)
#define ARG_NNL (arguments [3].value != NULL)
#define ARG_SPC (arguments [4].value != NULL)
#define ARG_SPA (arguments [5].value != NULL)







int main (int argc, char **argv)
{
  char buffer [BUFFER_SIZE];
  char *enc_table [255];
  char char_table [2 * sizeof (enc_table)];
  char *fsp, *dbs;
  int i, c;
  
  i = 0;
  for (c = 0; c < 255; c ++)
  {
    enc_table [c] = char_table + i;
    char_table [i ++] = (char) c;
    char_table [i ++] = 0;
  }
  i = arguments_parse ("htmlencode", "2 (2009-02-22)", "[text]", 
     "Converts plain text characters to HTML/XML entities.",
     argc, argv, arguments);
  if (ARG_NNL)
    enc_table ['\n'] = "";
  if (ARG_BR)
  {
    if (ARG_BRX)
      arguments_error_freeform ("Can't use both XML and SGML entity forms.");
    if (ARG_NNL)
      enc_table ['\n'] = "<br>";
    else
      enc_table ['\n'] = "<br>\n";
  }
  if (ARG_BRX)
  {
    if (ARG_NNL)
      enc_table ['\n'] = "<br />";
    else
      enc_table ['\n'] = "<br />\n";
  }
  if (!ARG_CR)
    enc_table ['\r'] = "";
  if (ARG_SPC)
    dbs = "&nbsp;";
  else
    dbs = " ";
  if (ARG_SPA)
  {
    if (ARG_SPC)
      arguments_error_freeform ("Options -s and -S are incompatible.");
    fsp = dbs = "&nbsp;";
  }
  else
    fsp = " ";
  enc_table [' '] = fsp;
  enc_table ['"'] = "&quot;";
  enc_table ['&'] = "&amp;";
  enc_table ['<'] = "&lt;";
  enc_table ['>'] = "&gt;";
  
  if (i == argc)
  {
loop1:
    i = read (STDIN_FILENO, buffer, BUFFER_SIZE - 1);
    CHECK_ERROR (i < 0, "read");
    if (i == 0)
      return EXIT_SUCCESS;
    for (c = 0; c < i; c ++)
    {
      CHECK_ERROR (printf (enc_table [(unsigned char) buffer [c]]) < 0,
         "printf");
      if (buffer [c] == ' ')
        enc_table [' '] = dbs;
      else
        enc_table [' '] = fsp;
    }
    goto loop1;
  }
  
loop2:
  for (c = 0; argv [i][c] != 0; c ++)
  {
    CHECK_ERROR (printf (enc_table [(unsigned char) argv [i][c]]) < 0,
       "printf");
    if (argv [i][c] == ' ')
      enc_table [' '] = dbs;
    else
      enc_table [' '] = fsp;
  }
  i ++;
  if (i == argc)
    return EXIT_SUCCESS;
  CHECK_ERROR (printf (enc_table [' ']) < 0, "printf");
  enc_table [' '] = dbs;
  goto loop2;
}
